@@ -1,10 +1,14 @@
 /* Released under AGPL v3 with exception for the OpenSSL library. See license.txt */
 
+#include "config.h"
+#include <libintl.h>
 #define _GNU_SOURCE
+#define __USE_XOPEN
+#include <time.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
 #include <locale.h>
-#include <libintl.h>
+#include "gen.h"
 #include <stdlib.h>
 #include <errno.h>
 #include <math.h>
@@ -23,12 +27,11 @@
 #include "mssl.h"
 #endif
 #include <arpa/inet.h>
-#include <time.h>
 #include <sys/time.h>
 #if defined(sun) || defined(__sun)
 #include <sys/termios.h>
 #endif
-#ifdef NC
+#if HAVE_NCURSES
 #include <ncurses.h>
 #endif
 
@@ -42,7 +45,7 @@
 #include "utils.h"
 #include "error.h"
 #include "socks5.h"
-#ifdef NC
+#if HAVE_NCURSES
 #include "nc.h"
 #endif
 #include "cookies.h"
@@ -107,7 +110,7 @@ void determine_terminal_size(int *max_y, int *max_x)
 
 void emit_statuslines(double run_time)
 {
-#ifdef NC
+#if HAVE_NCURSES
 	if (ncurses_mode)
 	{
 		time_t t = time(NULL);
@@ -126,12 +129,14 @@ void emit_statuslines(double run_time)
 
 void emit_headers(char *in)
 {
-#ifdef NC
-	static char shown = 0;
+#if HAVE_NCURSES
 	int len_in = -1;
 
-	if (!shown && ncurses_mode && in != NULL && (len_in = strlen(in) - 4) > 0)
+	if (ncurses_mode && in != NULL && (len_in = strlen(in) - 4) > 0)
 	{
+		extern WINDOW *w_slow;
+		werase(w_slow);
+
 		int pos = 0, pos_out = 0;
 		char *copy = (char *)malloc(len_in + 1), *dummy = NULL;
 
@@ -151,8 +156,6 @@ void emit_headers(char *in)
 		slow_log("\n%s", copy);
 
 		free(copy);
-
-		shown = 1;
 	}
 #else
 	(void)in;
@@ -233,7 +236,7 @@ void emit_error(int verbose, int seq, double start_ts)
 {
 	char *ts = show_ts ? get_ts_str(verbose) : NULL;
 
-#ifdef NC
+#if HAVE_NCURSES
 	if (ncurses_mode)
 	{
 		slow_log("\n%s%s", ts ? ts : "", get_error());
@@ -256,7 +259,7 @@ void emit_error(int verbose, int seq, double start_ts)
 
 void handler(int sig)
 {
-#ifdef NC
+#if HAVE_NCURSES
 	if (sig == SIGWINCH)
 		win_resize = 1;
 	else
@@ -430,10 +433,13 @@ void interpret_url(const char *in, char **path, char **hostname, int *portnr, ch
 		*dummy = 0x00;
 
 	/* fetch port number */
-	if (use_ssl || strncasecmp(in, "https://", 8) == 0)
-		*portnr = 443;
-	else
-		*portnr = 80;
+	if (*portnr == 0)
+	{
+		if (use_ssl || strncasecmp(in, "https://", 8) == 0)
+			*portnr = 443;
+		else
+			*portnr = 80;
+	}
 
 	if (!use_ipv6)
 	{
@@ -564,7 +570,7 @@ void do_aggregates(double cur_ms, int cur_ts, int n_aggregates, aggregate_t *agg
 
 			str_add(&line, " ms");
 
-#ifdef NC
+#if HAVE_NCURSES
 			if (ncurses_mode)
 				slow_log("\n%s", line);
 			else
@@ -865,7 +871,10 @@ void stats_line(const char with_header, const char *const complete_url, const in
 
 	if (ok > 0)
 	{
-		printf(gettext("round-trip min/avg/max%s = %s%.1f%s/%s%.1f%s/%s%.1f%s"), verbose ? gettext("/sd") : "", c_bright, t_total -> min, c_normal, c_bright, avg_httping_time, c_normal, c_bright, t_total -> max, c_normal);
+		if (t_total->calc_median)
+			printf(gettext("round-trip min/avg/median/max%s = %s%.1f%s/%s%.1f%s/%.1f/%s%.1f%s"), verbose ? gettext("/sd") : "", c_bright, t_total -> min, c_normal, c_bright, avg_httping_time, c_normal, calc_median(t_total), c_bright, t_total -> max, c_normal);
+		else
+			printf(gettext("round-trip min/avg/max%s = %s%.1f%s/%s%.1f%s/%s%.1f%s"), verbose ? gettext("/sd") : "", c_bright, t_total -> min, c_normal, c_bright, avg_httping_time, c_normal, c_bright, t_total -> max, c_normal);
 
 		if (verbose)
 		{
@@ -887,7 +896,7 @@ int main(int argc, char *argv[])
 	char *hostname = NULL;
 	char *proxy_host = NULL, *proxy_user = NULL, *proxy_password = NULL;
 	int proxy_port = 8080;
-	int portnr = 80;
+	int portnr = 0;
 	char *get = NULL, *request = NULL;
 	int req_len = 0;
 	int c = 0;
@@ -962,6 +971,8 @@ int main(int argc, char *argv[])
 	struct addrinfo *ai = NULL, *ai_use = NULL;
 	struct addrinfo *ai_proxy = NULL, *ai_use_proxy = NULL;
 	char http2 = 0;
+	char use_median = 0;
+	char ignore_ssl_errors = 0;
 
 	static struct option long_options[] =
 	{
@@ -984,6 +995,7 @@ int main(int argc, char *argv[])
 		{"show-kb",	0, NULL, 'X' },
 		{"no-cache",	0, NULL, 'Z' },
 #ifndef NO_SSL
+		{"insecure",	0, NULL, 'k' },
 		{"use-ssl",	0, NULL, 'l' },
 		{"show-fingerprint",	0, NULL, 'z' },
 #endif
@@ -1034,10 +1046,11 @@ int main(int argc, char *argv[])
 		{"tos", 1, NULL, 24 },
 		{"header", 1, NULL, 25 },
 		{"ca-path", 1, NULL, 26 },
-#ifdef NC
+		{"median", 0, NULL, 27 },
+#if HAVE_NCURSES
 		{"ncurses",	0, NULL, 'K' },
 		{"gui",	0, NULL, 'K' },
-#ifdef FW
+#if HAVE_FFTW3
 		{"no-graph",	0, NULL, 'D' },
 #endif
 #endif
@@ -1055,25 +1068,11 @@ int main(int argc, char *argv[])
 	bindtextdomain("httping", LOCALEDIR);
 	textdomain("httping");
 
-	init_statst(&t_resolve);
-	init_statst(&t_connect);
-	init_statst(&t_write);
-	init_statst(&t_request);
-	init_statst(&t_total);
-	init_statst(&t_ssl);
-	init_statst(&t_close);
-
-	init_statst(&stats_to);
-#if defined(linux) || defined(__FreeBSD__)
-	init_statst(&tcp_rtt_stats);
-#endif
-	init_statst(&stats_header_size);
-
 	determine_terminal_size(&max_y, &max_x);
 
 	signal(SIGPIPE, SIG_IGN);
 
-	while((c = getopt_long(argc, argv, "2DKEA5MvYWT:ZQ6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:zP:U:C:F", long_options, NULL)) != -1)
+	while((c = getopt_long(argc, argv, "2DKEA5MvYWT:ZQ6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:zP:U:C:Fk", long_options, NULL)) != -1)
 	{
 		switch(c)
 		{
@@ -1085,6 +1084,10 @@ int main(int argc, char *argv[])
 				ca_path = optarg;
 				break;
 
+			case 27:
+				use_median = 1;
+				break;
+
 			case 25:
 				add_header(&additional_headers, &n_additional_headers, optarg);
 				break;
@@ -1129,7 +1132,7 @@ int main(int argc, char *argv[])
 				use_tcp_nodelay = 0;
 				break;
 
-#ifdef NC
+#if HAVE_NCURSES
 			case 14:
 				draw_phase = 1;
 				break;
@@ -1147,14 +1150,14 @@ int main(int argc, char *argv[])
 				graph_limit = atof(optarg);
 				break;
 
-#ifdef NC
+#if HAVE_NCURSES
 			case 'K':
 				ncurses_mode = 1;
 				adaptive_interval = 1;
 				if (!wait_set)
 					wait = 0.5;
 				break;
-#ifdef FW
+#if HAVE_FFTW3
 			case 'D':
 				nc_graph = 0;
 				break;
@@ -1359,6 +1362,10 @@ int main(int argc, char *argv[])
 				show_statuscodes = 1;
 				break;
 
+			case 'k':
+				ignore_ssl_errors = 1;
+				break;
+
 			case 'V':
 				version();
 				return 0;
@@ -1391,11 +1398,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'F':
-#ifdef TCP_TFO
 				use_tfo = 1;
-#else
-				fprintf(stderr, gettext("Warning: TCP TFO is not supported. Disabling.\n"));
-#endif
 				break;
 
 			case 22:
@@ -1416,6 +1419,20 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	init_statst(&t_resolve, use_median);
+	init_statst(&t_connect, use_median);
+	init_statst(&t_write, use_median);
+	init_statst(&t_request, use_median);
+	init_statst(&t_total, use_median);
+	init_statst(&t_ssl, use_median);
+	init_statst(&t_close, use_median);
+
+	init_statst(&stats_to, use_median);
+#if defined(linux) || defined(__FreeBSD__)
+	init_statst(&tcp_rtt_stats, use_median);
+#endif
+	init_statst(&stats_header_size, use_median);
+
 	if (do_fetch_proxy_settings)
 		fetch_proxy_settings(&proxy_user, &proxy_password, &proxy_host, &proxy_port, use_ssl, use_ipv6);
 
@@ -1451,7 +1468,7 @@ int main(int argc, char *argv[])
 	if (!auth_password)
 		auth_password = ap_dummy;
 
-#ifdef NC
+#if HAVE_NCURSES
 	if (ncurses_mode)
 	{
 		if (wait == 0.0)
@@ -1464,7 +1481,7 @@ int main(int argc, char *argv[])
 	if (strncmp(complete_url, "https://", 8) == 0 && !use_ssl)
 	{
 		use_ssl = 1;
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 		{
 			slow_log(gettext("\nAuto enabling SSL due to https-URL\n"));
@@ -1482,7 +1499,7 @@ int main(int argc, char *argv[])
 
 	if (verbose)
 	{
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 		{
 			slow_log(gettext("\nConnecting to host %s, port %d and requesting file %s"), hostname, portnr, get);
@@ -1514,7 +1531,7 @@ int main(int argc, char *argv[])
 
 	if (!quiet && !machine_readable && !nagios_mode && !json_output)
 	{
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 			slow_log("\nPING %s:%d (%s):", hostname, portnr, get);
 		else
@@ -1549,7 +1566,7 @@ int main(int argc, char *argv[])
 	started_at = get_ts();
 	if (proxy_host)
 	{
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 		{
 			slow_log(gettext("\nResolving hostname %s"), proxy_host);
@@ -1567,7 +1584,7 @@ int main(int argc, char *argv[])
 	else if (resolve_once)
 	{
 		char *res_host = divert_connect ? divert_connect : hostname;
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 		{
 			slow_log(gettext("\nResolving hostname %s"), res_host);
@@ -1649,7 +1666,7 @@ persistent_loop:
 
 				memset(&addr, 0x00, sizeof addr);
 
-#ifdef NC
+#if HAVE_NCURSES
 				if (ncurses_mode && first_resolve)
 				{
 					slow_log(gettext("\nResolving hostname %s"), res_host);
@@ -1718,7 +1735,7 @@ persistent_loop:
 
 				if (line)
 				{
-#ifdef NC
+#if HAVE_NCURSES
 					if (ncurses_mode)
 						slow_log("\n%s", line);
 					else
@@ -1783,7 +1800,7 @@ persistent_loop:
 #ifndef NO_SSL
 				if (use_ssl && ssl_h == NULL)
 				{
-					int rc = connect_ssl(fd, client_ctx, &ssl_h, &s_bio, timeout, &ssl_handshake, hostname);
+					int rc = connect_ssl(fd, client_ctx, &ssl_h, &s_bio, timeout, &ssl_handshake, hostname, ignore_ssl_errors);
 					if (rc == 0)
 						update_statst(&t_ssl, ssl_handshake);
 					else
@@ -1901,7 +1918,7 @@ persistent_loop:
 			}
 
 #if defined(linux) || defined(__FreeBSD__)
-#ifdef NC
+#if HAVE_NCURSES
 			if (!use_ssl && ncurses_mode)
 			{
 				struct timeval tv;
@@ -1957,7 +1974,7 @@ persistent_loop:
 
 			rc = get_HTTP_headers(fd, ssl_h, &reply, &overflow, timeout);
 
-#ifdef NC
+#if HAVE_NCURSES
 			if (ncurses_mode && !get_instead_of_head && overflow > 0)
 			{
 				static int more_data_cnt = 0;
@@ -2142,7 +2159,7 @@ persistent_loop:
 #if defined(linux) || defined(__FreeBSD__)
 			if (getsockopt(fd, IPPROTO_TCP, TCP_INFO, &info, &info_len) == 0)
 			{
-#ifdef TCP_TFO
+#ifdef TCPI_OPT_SYN_DATA
 				if (info.tcpi_options & TCPI_OPT_SYN_DATA)
 					tfo_success = 1;
 #endif
@@ -2323,19 +2340,17 @@ persistent_loop:
 
 				if (audible)
 				{
-#ifdef NC
+#if HAVE_NCURSES
 					my_beep();
 #else
 					putchar('\a');
 #endif
 				}
 
-#ifdef TCP_TFO
 				if (tfo_success)
 					str_add(&line, " F");
-#endif
 
-#ifdef NC
+#if HAVE_NCURSES
 				if (ncurses_mode)
 				{
 					if (dummy_ms >= show_slow_log)
@@ -2376,7 +2391,7 @@ persistent_loop:
 		}
 
 		emit_statuslines(get_ts() - started_at);
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 			update_stats(&t_resolve, &t_connect, &t_request, &t_total, &t_ssl, curncount, err, sc, fp, use_tfo, nc_graph, &stats_to, &tcp_rtt_stats, re_tx, pmtu, recv_tos, &t_close, &t_write, n_dynamic_cookies, abbreviate, &stats_header_size);
 #endif
@@ -2384,7 +2399,7 @@ persistent_loop:
 		free(sc);
 		free(fp);
 
-#ifdef NC
+#if HAVE_NCURSES
 		if (ncurses_mode)
 			update_terminal();
 		else
@@ -2422,7 +2437,7 @@ persistent_loop:
 #endif
 	}
 
-#ifdef NC
+#if HAVE_NCURSES
 	if (ncurses_mode)
 		end_ncurses();
 #endif
@@ -2471,6 +2486,19 @@ error_exit:
 
 	fflush(NULL);
 
+	uninit_statst(&t_resolve);
+	uninit_statst(&t_connect);
+	uninit_statst(&t_write);
+	uninit_statst(&t_request);
+	uninit_statst(&t_total);
+	uninit_statst(&t_ssl);
+	uninit_statst(&t_close);
+	uninit_statst(&stats_to);
+#if defined(linux) || defined(__FreeBSD__)
+	uninit_statst(&tcp_rtt_stats);
+#endif
+	uninit_statst(&stats_header_size);
+
 	if (ok)
 		return 0;
 
