@@ -1,6 +1,8 @@
+#include "config.h"
+#if HAVE_NCURSES
 #define _GNU_SOURCE
 #include <stdio.h>
-#include <libintl.h>
+#include "gen.h"
 #include <math.h>
 #include <poll.h>
 #include <stdarg.h>
@@ -8,6 +10,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/ioctl.h>
+#include <unistd.h>
 
 #include <ncurses.h>
 
@@ -16,7 +19,7 @@
 #include "gen.h"
 #include "main.h"
 #include "kalman.h"
-#ifdef FW
+#if HAVE_FFTW3
 #include "fft.h"
 #endif
 #include "utils.h"
@@ -28,6 +31,7 @@ int logs_n = 0, slow_n = 0, fast_n = 0;
 char **slow_history = NULL, **fast_history = NULL;
 int window_history_n = 0;
 char use_colors = 0;
+int fast_nr = 0;
 
 double graph_limit = MY_DOUBLE_INF;
 double hz = 1.0;
@@ -38,6 +42,8 @@ int history_n = 0;
 
 char draw_phase = 0;
 
+char fancy_colors = 0;
+
 char pause_graphs = 0;
 
 typedef enum { C_WHITE = 0, C_GREEN, C_YELLOW, C_BLUE, C_MAGENTA, C_CYAN, C_RED } color_t;
@@ -65,7 +71,7 @@ void create_windows(void)
 		delwin(w_fast);
 	}
 
-#ifdef FW
+#if HAVE_FFTW3
 	fft_free();
 	fft_init(max_x);
 #endif
@@ -291,6 +297,8 @@ void init_ncurses_ui(double graph_limit_in, double hz_in, char use_colors_in)
         init_pair(C_GREEN, COLOR_GREEN, COLOR_BLACK);
         init_pair(C_RED, COLOR_RED, COLOR_BLACK);
 
+	fancy_colors = COLORS >= 256 && COLOR_PAIRS >= 256 && can_change_color();
+
 	kalman_init(0.0);
 
         recreate_terminal();
@@ -323,7 +331,7 @@ void end_ncurses(void)
 	free(history);
 	free(history_set);
 
-#ifdef FW
+#if HAVE_FFTW3
 	fft_free();
 	fft_stop();
 #endif
@@ -334,6 +342,7 @@ void end_ncurses(void)
 
 void fast_log(const char *fmt, ...)
 {
+	int col_nr = fast_nr++ % 3;
 	char buffer[4096] = { 0 };
         va_list ap;
 
@@ -345,8 +354,18 @@ void fast_log(const char *fmt, ...)
 	memmove(&fast_history[1], &fast_history[0], (window_history_n - 1) * sizeof(char *));
 	fast_history[0] = strdup(buffer);
 
+	if (col_nr == 0)
+		wattron(w_fast, A_DIM);
+	else if (col_nr == 2)
+		wattron(w_fast, A_BOLD);
+
 	myprint(w_fast, buffer);
 
+	if (col_nr == 0)
+		wattroff(w_fast, A_DIM);
+	else if (col_nr == 2)
+		wattroff(w_fast, A_BOLD);
+
 	if (win_resize)
 		recreate_terminal();
 }
@@ -491,7 +510,7 @@ double get_cur_scc()
 	return ((double)n * t[0] - t[1]) / scc_val;
 }
 
-#ifdef FW
+#if HAVE_FFTW3
 void draw_fft(void)
 {
 	double mx_mag = 0.0;
@@ -711,6 +730,9 @@ void draw_graph(double val)
 
 void show_stats_t(int y, int x, char *header, stats_t *data, char abbreviate)
 {
+	if (y & 1)
+		wattron(w_stats, A_DIM);
+
 	if (data -> valid)
 	{
 		char *cur_str = format_value(data -> cur, 6, 2, abbreviate);
@@ -733,6 +755,9 @@ void show_stats_t(int y, int x, char *header, stats_t *data, char abbreviate)
 	{
 		myprintloc(w_stats, y, x, gettext("%s:    n/a"), header);
 	}
+
+	if (y & 1)
+		wattroff(w_stats, A_DIM);
 }
 
 void update_stats(stats_t *resolve, stats_t *connect, stats_t *request, stats_t *total, stats_t *ssl_setup, int n_ok, int n_fail, const char *last_connect_str, const char *fp, char use_tfo, char dg, stats_t *st_to, stats_t *tcp_rtt_stats, int re_tx, int pmtu, int tos, stats_t *close_st, stats_t *t_write, int n_cookies, char abbreviate, stats_t *stats_header_size)
@@ -832,7 +857,7 @@ void update_stats(stats_t *resolve, stats_t *connect, stats_t *request, stats_t
 	if (dg && !pause_graphs)
 	{
 		draw_graph(k);
-#ifdef FW
+#if HAVE_FFTW3
 		draw_fft();
 #endif
 	}
@@ -842,3 +867,4 @@ void update_stats(stats_t *resolve, stats_t *connect, stats_t *request, stats_t
 	if (win_resize || force_redraw)
 		recreate_terminal();
 }
+#endif
